import 'package:analyzer/dart/ast/ast.dart';
import 'package:welltested/src/extensions/annotation_ext.dart';
import 'package:welltested/src/models/test_request/widget/widget_test_request.dart';
import 'package:welltested/src/utilities/utilities.dart';
import 'package:welltested/src/visitors/function_body_visitor.dart';
import 'package:welltested/src/visitors/stateful_widget_visitor.dart';
import 'package:welltested_annotation/welltested_annotation.dart';

class WidgetTestExecutor {
  Future<WidgetTestRequest> prepareRequest({
    required ClassDeclaration classDeclaration,
    required final CompilationUnit compilationUnit,
  }) async {
    final element = classDeclaration.declaredElement;
    final annotation = classDeclaration.declaredElement!.metadata.firstWhere(
        (elementAnnotation) =>
            elementAnnotation.element!.displayName ==
            Welltested().runtimeType.toString());

    final supportingClasses = annotation.supportingClasses ?? [];
    final StatefulWidgetFinder statefulWidgetFinder = StatefulWidgetFinder();
    classDeclaration.visitChildren(statefulWidgetFinder);

    final List<ClassDeclaration> fullWidgetDetail = [];
    for (final classElement in statefulWidgetFinder.classElements) {
      fullWidgetDetail.add((await getResolvedAstNodeForElement(classElement))
          as ClassDeclaration);
    }
    fullWidgetDetail.add(classDeclaration); // add the parent class as well

    final functionBodyVisitor = FunctionBodyVisitor();
    fullWidgetDetail.forEach((w) => w.visitChildren(functionBodyVisitor));

    final variables = functionBodyVisitor.updateVariableNameNType;

    return WidgetTestRequest(
      widgetName: classDeclaration.name.toString(),
      fileImportPath: element!.source.uri.toString(),
      supportingClasses: supportingClasses,
      classDeclaration: classDeclaration,
      compilationUnit: compilationUnit,
      variableDeclarations: variables,
      fullWidgetDetail: fullWidgetDetail,
      callFlows: functionBodyVisitor.callFlows,
      classElements: functionBodyVisitor.classElements,
      enumElements: functionBodyVisitor.enumElements.toList(),
      staticMethods: functionBodyVisitor.staticMethods,
      methodInvocations: functionBodyVisitor.nestedMethods,
      simpleIdentifiers: functionBodyVisitor.simpleIdentifiers.toList(),
      typeArguments: functionBodyVisitor.typeArguments.toList(),
    );
  }
}
