/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */

import 'dart:io';

import 'package:welltested/src/executors/fixer_executor.dart';
import 'package:welltested/src/repositories/dart_server_repository.dart';
import 'package:welltested/src/repositories/writer_repository.dart';

class WriterExecutor {
  final WriterRepository writerRepository;
  final DartServerRepository dartServerRepository;
  final FixerExecutor fixerExecutor;

  WriterExecutor(
      this.writerRepository, this.dartServerRepository, this.fixerExecutor);

  Future<File> processGeneration(
      String path, String content, String methodKey) async {
    await writerRepository.outputGeneratedFile(path, content);

    final updatedContent =
        await fixerExecutor.findAndFixIssues(path, methodKey);
    return outputGeneration(path, updatedContent);
  }

  Future<File> processHelperFunctionGeneration(
    String path,
    String content,
  ) async {
    content = _addImports(content);
    await writerRepository.outputGeneratedFile(path, content);
    final updatedContent =
        await fixerExecutor.fixIssuesForHelperFunctions(path);
    return outputGeneration(path, updatedContent);
  }

  String _addImports(String fileContent) {
    String importTest = "import 'package:flutter_test/flutter_test.dart';";
    if (!fileContent.contains(importTest)) {
      fileContent = importTest + fileContent;
    }
    return fileContent;
  }

  Future<File> outputGeneration(
    String path,
    String content,
  ) async {
    return await writerRepository.outputGeneratedFile(path, content);
  }
}
