import 'package:analyzer/dart/analysis/utilities.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:welltested/src/visitors/integration_test_visitor.dart';

extension AstNodeExt on AstNode {
  bool get firstLayerToMain {
    return parent?.parent?.parent?.parent is FunctionDeclaration &&
        (parent?.parent?.parent?.parent as FunctionDeclaration).name.value() ==
            'main';
  }
}

extension VisitVisitorExt on String {
  IntegrationTestVisitor executedVisitor(String path, String flow) {
    final unit = parseString(content: this, path: path).unit;
    final integrationTestVisitor =
        IntegrationTestVisitor(flow: flow, sourceCode: this);
    integrationTestVisitor.visitCompilationUnit(unit); //gets main, imports
    integrationTestVisitor.main?.functionExpression.body.visitChildren(
        integrationTestVisitor); //Gets flows, and method declarations inside main.Due to some reason, recursive ast visitor is not able to extract all at once
    // TODO: Flows and declarations inside group.
    //TODO: Method Declarations outside main
    return integrationTestVisitor;
  }
}

extension IntegrationTestVisitorExt on IntegrationTestVisitor {
  MethodInvocation? matchingFlow(String? flowDescription) {
    if (flowDescription == null) throw "Flow Description cannot be null";
    final matchingFlowIndex = allFlows.indexWhere(
        (existingFlow) => existingFlow.flowDescription == flowDescription);
    if (matchingFlowIndex != -1) return allFlows[matchingFlowIndex];
    return null;
  }

  FunctionDeclarationStatement? matchingMethods(String methodName) {
    final matchingMethodIndex = methodsDeclared
        .indexWhere((existingMethod) => existingMethod.name == methodName);
    if (matchingMethodIndex != -1) return methodsDeclared[matchingMethodIndex];
    return null;
  }
}

extension FlowExtension on MethodInvocation {
  String? get flowDescription {
    return (argumentList.arguments.first as StringLiteral).stringValue;
  }
}

extension MethodExtension on FunctionDeclarationStatement {
  String get name => this.functionDeclaration.name.value() as String;
}
