/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';
import 'dart:typed_data';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:http/io_client.dart';
import 'package:welltested/src/models/test_request/base_test_request.dart';
import 'package:welltested/src/models/test_request/integration/integration_test_request.dart';
import 'package:welltested/src/models/test_request/integration/integration_update_test_request.dart';
import 'package:welltested/src/models/test_request/widget/widget_test_request.dart';
import 'package:welltested/src/utilities/env_parser.dart';
import 'package:welltested/src/utilities/logging.dart';

import '../utilities/agent/extraction_agent.dart';

import 'package:welltested/src/version.dart';

class GenerationRepository {
  final baseUrl = 'https://api.welltested.ai';

  Future<Map<String, dynamic>> buildUnitTests(
      UnitTestRequest testRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final response = await http.post(
          Uri.parse('$baseUrl/ai/unit-tests/generate'),
          body: testRequestData.toJson(),
          headers: {"apikey": apiKey, "cli_version": packageVersion});
      if (response.statusCode == 200) {
        return jsonDecode(response.body);
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> updateTest(
      UnitTestRequest testRequestData, String fixedTest) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final response = await http.post(
          Uri.parse('$baseUrl/ai/unit-tests/update'),
          body: testRequestData.toJson()..['fixed_test_case'] = fixedTest,
          headers: {"apikey": apiKey, "cli_version": packageVersion});
      if (response.statusCode == 200) {
        return jsonDecode(response.body);
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      throw (e);
    }
  }

  Future<Map<String, dynamic>> testcases(
      UnitTestRequest testRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final response = await http.post(
          Uri.parse('$baseUrl/ai/unit-tests/test-cases'),
          body: testRequestData.toJson(),
          headers: {"apikey": apiKey, "cli_version": packageVersion});
      if (response.statusCode == 200) {
        return jsonDecode(response.body);
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      throw (e);
    }
  }

  Stream<String> dryRun(UnitTestRequest testRequestData) async* {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final url = '$baseUrl/ai/unit-tests/dry-run';

      HttpClient httpClient = HttpClient();
      IOClient ioClient = IOClient(httpClient);
      Uri uri = Uri.parse(url);
      final request = http.Request("POST", uri);
      request.bodyFields = testRequestData.toJson().cast<String, String>();
      request.headers['apikey'] = apiKey;
      request.headers['cli_version'] = packageVersion;
      final response = await ioClient.send(request);
      if (response.statusCode == 200) {
        final transformer = StreamTransformer<Uint8List, String>.fromHandlers(
          handleData: (data, sink) {
            String text = utf8.decoder.convert(data);
            List<String> lines = text.split('\n');
            for (String line in lines) {
              sink.add(line);
            }
          },
        );
        await for (String line
            in response.stream.cast<Uint8List>().transform(transformer)) {
          yield line;
        }
      } else {
        if (response.statusCode == 403) {
          throw ("🚨 Invalid Welltested API Key. Use the key recieved in the registered mail.");
        } else {
          //TODO: Replace this with a clear response from backend.
          throw ("Unknown Exception. Please reach out to team@welltested.ai for resolution");
        }
      }
    } catch (e) {
      throw (e);
    }
  }

  Stream<String> checkTestability(UnitTestRequest testRequestData) async* {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final url = '$baseUrl/ai/unit-tests/testability';

      HttpClient httpClient = HttpClient();
      IOClient ioClient = IOClient(httpClient);
      Uri uri = Uri.parse(url);
      final request = http.Request("POST", uri);
      request.bodyFields = testRequestData.toJson().cast<String, String>();
      request.headers['apikey'] = apiKey;
      request.headers['cli_version'] = packageVersion;
      final response = await ioClient.send(request);
      if (response.statusCode == 200) {
        final transformer = StreamTransformer<Uint8List, String>.fromHandlers(
          handleData: (data, sink) {
            String text = utf8.decoder.convert(data);
            List<String> lines = text.split('\n');
            for (String line in lines) {
              sink.add(line);
            }
          },
        );
        await for (String line
            in response.stream.cast<Uint8List>().transform(transformer)) {
          yield line;
        }
      } else {
        if (response.statusCode == 403) {
          throw ("🚨 Invalid Welltested API Key. Use the key recieved in the registered mail.");
        } else {
          //TODO: Replace this with a clear response from backend.
          throw ("Unknown Exception. Please reach out to team@welltested.ai for resolution");
        }
      }
    } catch (e) {
      throw (e);
    }
  }

  // WIDGET TESTS
  Future<List<ExtractedCode>> getRequirements(
      WidgetTestRequest testRequestData) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final response = await http.post(
          Uri.parse('$baseUrl/ai/widget-tests/extract'),
          body: jsonEncode(testRequestData.toJson()),
          headers: {"apikey": apiKey, "Content-Type": "application/json"});

      if (response.statusCode == 200) {
        final responseList = jsonDecode(response.body);
        final List<ExtractedCode> extractedCodeList = [];
        for (var item in responseList) {
          extractedCodeList.add(ExtractedCode.fromRequirment(item));
        }
        return extractedCodeList;
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      rethrow;
    }
  }

  Future<Map<String, dynamic>> buildWidgettests(
    WidgetTestRequest testRequestData,
  ) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      if (wtLog.verbose) testRequestData.printData();

      final response = await http.post(
          Uri.parse('$baseUrl/ai/widget-tests/generate'),
          body: jsonEncode(testRequestData.toJson()),
          headers: {"apikey": apiKey, "Content-Type": "application/json"});

      if (response.statusCode == 200) {
        return jsonDecode(response.body);
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      rethrow;
    }
  }

  Future<Map<String, dynamic>> updateWidgetTest(
      WidgetTestRequest testRequestData, String fixedTest) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final body = testRequestData.toJson()..['fixed_test_case'] = fixedTest;
      final response = await http.post(
          Uri.parse('$baseUrl/ai/widget-tests/update'),
          body: jsonEncode(body),
          headers: {"apikey": apiKey, "Content-Type": "application/json"});
      if (response.statusCode == 200) {
        return jsonDecode(response.body);
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      rethrow;
    }
  }

  Future<Map<String, dynamic>> buildIntegrationTests(
    IntegrationRequestData testRequestData,
  ) async {
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }

      final queue_init_response = await http.post(
          Uri.parse('$baseUrl/orchestrated/ai/integration-tests/start'),
          body: jsonEncode(testRequestData.toJson()),
          headers: {
            "apikey": apiKey,
            "Content-Type": "application/json",
          });

      if (queue_init_response.statusCode == 200) {
        final init_data = jsonDecode(queue_init_response.body);
        final stopwatch = Stopwatch()..start();

        while (stopwatch.elapsedMilliseconds < init_data['timeout']) {
          await Future.delayed(
              Duration(milliseconds: init_data['poll_interval']));
          final status_response = await http.get(
              Uri.parse(
                  '$baseUrl/orchestrated/ai/integration-tests/${init_data['orchestration_id']}/status'),
              headers: {
                "apikey": apiKey,
                "Content-Type": "application/json",
              });
          if (status_response.statusCode == 200) {
            final status_data = jsonDecode(status_response.body);
            if (status_data['status'] == 'failed') {
              throw (status_data['error_message']);
            } else if (status_data['status'] == 'Completed') {
              return status_data['result'] as Map<String, dynamic>;
            } else {
              continue;
            }
          } else {
            throw (_handleErrorStatusCodes(status_response));
          }
        }
        throw ('Integration Test generation process timed out');
      } else {
        throw (_handleErrorStatusCodes(queue_init_response));
      }
    } catch (e) {
      rethrow;
    }
  }

  Future<String> updateIntegrationTests(
    IntegrationUpdateRequestData testRequestData,
    String fixedTest,
  ) async {
    final json = testRequestData.toJson();
    log(jsonEncode(json));
    try {
      final DotEnv env = DotEnv(includePlatformEnvironment: true)..load();
      final apiKey = env.getOrElse("WELLTESTED_API", () => '');
      if (apiKey.isEmpty) {
        throw "🚨 WELLTESTED_API key not found. Please refer documentation for more Info.";
      }
      final body = testRequestData.toJson()..['fixed_test'] = fixedTest;
      final response = await http.post(
          Uri.parse('$baseUrl/ai/integration-tests/update'),
          body: jsonEncode(body),
          headers: {"apikey": apiKey, "Content-Type": "application/json"});

      if (response.statusCode == 200) {
        return response.body;
      } else {
        throw (_handleErrorStatusCodes(response));
      }
    } catch (e) {
      rethrow;
    }
  }

  //TODO: Check Error Scenarios
  String _handleErrorStatusCodes(Response response) {
    final errorMessage = jsonDecode(response.body)['error'];
    if (response.statusCode == 403) {
      if (errorMessage == 'Unable to process request. CLI version is outdated')
        return "🚨 ${errorMessage}";

      return "🚨 Invalid Welltested API Key. Use the key recieved in the registered mail.";
    } else if (response.statusCode == 500) {
      return "🚨 Error: ${errorMessage}";
    } else if (response.statusCode == 503) {
      return "🚨 Error: ${errorMessage}";
    } else {
      return "Unknown Exception. Please reach out to team@welltested.ai for resolution. Details: ${response.body}";
    }
  }
}
