/*
 * Created on October 5th 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */

import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';

class ClassAstVistor extends RecursiveAstVisitor {
  Map<String, MethodInvocation> methodInvocation = {};
  Map<String, InstanceCreationExpression> instanceCreationExpressions = {};
  List<ClassDeclaration> classDeclarations = [];
  List<NamedType> namedTypes = [];
  List<MethodDeclaration> methodDeclarations = [];

  @override
  visitMethodInvocation(MethodInvocation node) {
    methodInvocation[node.methodName.toString()] = node;
    return super.visitMethodInvocation(node);
  }

  @override
  visitMethodDeclaration(MethodDeclaration node) {
    methodDeclarations.add(node);
    return super.visitMethodDeclaration(node);
  }

  @override
  visitClassDeclaration(ClassDeclaration node) {
    classDeclarations.add(node);
    return super.visitClassDeclaration(node);
  }

  @override
  visitConstructorReference(ConstructorReference node) {
    return super.visitConstructorReference(node);
  }

  @override
  visitFieldDeclaration(FieldDeclaration node) {
    final type = node.fields.type;
    if (type is NamedType) {
      _extractTypeArguments(type);
    }
    return super.visitFieldDeclaration(node);
  }

  void _extractTypeArguments(NamedType namedType) {
    final typeArguments = namedType.typeArguments?.arguments;
    if (typeArguments != null) {
      for (var typeArgument in typeArguments) {
        if (typeArgument is NamedType) {
          namedTypes.add(typeArgument);
        }
      }
    }
  }

  @override
  visitInstanceCreationExpression(InstanceCreationExpression node) {
    instanceCreationExpressions[node.constructorName.toString()] = node;
    return super.visitInstanceCreationExpression(node);
  }
}
