/*
 * Created on October 5th 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:welltested_annotation/welltested_annotation.dart';

class StatefulWidgetFinder extends RecursiveAstVisitor {
  List<ClassElement> classElements = [];

  @override
  visitMethodDeclaration(MethodDeclaration node) {
    // ? Can there be a better way to do this?
    if (node.name.toString() == 'createState') {
      final body = node.body;
      final functionVisitor = WidgetFunctionVisitor();
      body.accept(functionVisitor);
      for (final classElement in functionVisitor.classElements) {
        classElements.add(classElement);
      }
    }
    return super.visitMethodDeclaration(node);
  }
}

class WidgetFunctionVisitor extends RecursiveAstVisitor {
  final List<ClassElement> classElements = [];

  void addToClassElements(ClassElement model) {
    if (!classElements.map((e) => e.name).contains(model.name)) {
      classElements.add(model);
    }
  }

  @override
  visitSimpleIdentifier(SimpleIdentifier node) {
    final staticElement = node.staticElement;

    if (staticElement is ClassElement && !staticElement.library.isInSdk) {
      if (staticElement.name == Welltested().runtimeType.toString() ||
          staticElement.name == Testcases([]).runtimeType.toString()) return;
      addToClassElements(staticElement);
    }
    return super.visitSimpleIdentifier(node);
  }
}
